@@ -24,6 +24,7 @@
  */
 
 #include <wx/wx.h>
+#include <wx/clipbrd.h>
 #include <wx/config.h>
 #include <wx/fs_zip.h>
 #include <wx/image.h>
@@ -34,12 +35,14 @@
 #include <wx/stdpaths.h>
 #include <wx/filename.h>
 #include <wx/filedlg.h>
+#include <wx/snglinst.h>
 #include <wx/sysopt.h>
 #include <wx/stdpaths.h>
 #include <wx/aboutdlg.h>
 #include <wx/artprov.h>
 #include <wx/datetime.h>
 #include <wx/intl.h>
+#include <wx/ipc.h>
 #include <wx/translation.h>
 
 #ifdef __WXOSX__
@@ -62,6 +65,7 @@
 
 #include "edapp.h"
 #include "edframe.h"
+#include "concurrency.h"
 #include "manager.h"
 #include "prefsdlg.h"
 #include "extractor.h"
@@ -70,11 +74,13 @@
 #include "hidpi.h"
 #include "icons.h"
 #include "version.h"
+#include "str_helpers.h"
 #include "tm/transmem.h"
 #include "utility.h"
 #include "prefsdlg.h"
 #include "errors.h"
 #include "language.h"
+#include "crowdin_client.h"
 
 #ifdef __WXOSX__
 struct PoeditApp::RecentMenuData
@@ -85,7 +91,164 @@ struct PoeditApp::RecentMenuData
 };
 #endif
 
+#ifndef DONT_MIGRATE_LEGACY_TM
 extern bool MigrateLegacyTranslationMemory();
+#endif
+
+
+#ifndef __WXOSX__
+
+// IPC for ensuring that only one instance of Poedit runs at a time. This is
+// handled native on OS X and GtkApplication could do it under GTK+3, but wx
+// doesn't support that and we have to implement everything manually for both
+// Windows and GTK+ ports.
+
+namespace
+{
+
+#ifdef __UNIX__
+wxString GetRuntimeDir()
+{
+    wxString dir;
+    if (!wxGetEnv("XDG_RUNTIME_DIR", &dir))
+        dir = wxGetHomeDir();
+    if (dir.Last() != '/')
+        dir += '/';
+    return dir;
+}
+wxString RemoteService() { return GetRuntimeDir() + "poedit.ipc"; }
+#else
+wxString RemoteService() { return "Poedit"; }
+#endif
+
+const char *IPC_TOPIC = "cmdline";
+
+} // anonymous namespace
+
+class PoeditApp::RemoteServer
+{
+public:
+    RemoteServer(PoeditApp *app) : m_server(app)
+    {
+        m_server.Create(RemoteService());
+    }
+
+private:
+    class Connection : public wxConnection
+    {
+    public:
+        Connection(PoeditApp *app) : m_app(app) {}
+
+        bool OnExec(const wxString& topic, const wxString& data) override
+        {
+            if (topic != IPC_TOPIC)
+                return false;
+            wxString payload;
+            if (data == "Activate")
+            {
+                m_app->OpenNewFile();
+                return true;
+            }
+            if (data.StartsWith("OpenURI:", &payload))
+            {
+                m_app->HandleCustomURI(payload);
+                return true;
+            }
+            if (data.StartsWith("OpenFile:", &payload))
+            {
+                wxArrayString a;
+                a.push_back(payload);
+                m_app->OpenFiles(a);
+                return true;
+            }
+            return false;
+        }
+
+    private:
+        PoeditApp *m_app;
+    };
+
+    class Server : public wxServer
+    {
+    public:
+        Server(PoeditApp *app) : m_app(app) {}
+
+        wxConnectionBase *OnAcceptConnection(const wxString& topic) override
+        {
+            if (topic != IPC_TOPIC)
+                return nullptr;
+            return new Connection(m_app);
+        }
+
+    private:
+        PoeditApp *m_app;
+    };
+
+    Server m_server;
+};
+
+class PoeditApp::RemoteClient
+{
+public:
+    RemoteClient(wxSingleInstanceChecker *instCheck) : m_instCheck(instCheck), m_client()
+    {
+    }
+
+    enum ConnectionStatus
+    {
+        NoOtherInstance,
+        Connected,
+        Failure
+    };
+
+    ConnectionStatus ConnectIfNeeded()
+    {
+        // Try several times in case of a race condition where one instance
+        // is exiting and not communicating, while another one is just starting.
+        for (int attempt = 0; attempt < 3; attempt++)
+        {
+            if (!m_instCheck->IsAnotherRunning())
+                return NoOtherInstance;
+
+            m_conn.reset(m_client.MakeConnection("localhost", RemoteService(), IPC_TOPIC));
+            if (m_conn)
+                return Connected;
+
+            // else: Something went wrong, no connection despite other instance existing.
+            //       Try again in a little while.
+            wxMilliSleep(100);
+        }
+
+        return Failure;
+    }
+
+    void Activate()
+    {
+        Command("Activate");
+    }
+
+    void HandleCustomURI(const wxString& uri)
+    {
+        Command("OpenURI:" + uri);
+    }
+
+    void OpenFile(const wxString& filename)
+    {
+        wxFileName fn(filename);
+        fn.MakeAbsolute();
+        Command("OpenFile:" + fn.GetFullPath());
+    }
+
+private:
+    void Command(const wxString cmd) { m_conn->Execute(cmd); }
+
+    wxSingleInstanceChecker *m_instCheck;
+    wxClient m_client;
+    std::unique_ptr<wxConnectionBase> m_conn;
+};
+
+#endif // __WXOSX__
+
 
 IMPLEMENT_APP(PoeditApp);
 
@@ -105,6 +268,32 @@ wxString PoeditApp::GetAppVersion() const
     return wxString::FromAscii(POEDIT_VERSION);
 }
 
+wxString PoeditApp::GetAppBuildNumber() const
+{
+#if defined(__WXOSX__)
+    NSDictionary *infoDict = [[NSBundle mainBundle] infoDictionary];
+    NSString *ver = [infoDict objectForKey:@"CFBundleVersion"];
+    return str::to_wx(ver);
+#elif defined(__WXMSW__)
+    auto exe = wxStandardPaths::Get().GetExecutablePath();
+    DWORD unusedHandle;
+    DWORD fiSize = GetFileVersionInfoSize(exe.wx_str(), &unusedHandle);
+    if (fiSize == 0)
+        return "";
+    wxCharBuffer fi(fiSize);
+    if (!GetFileVersionInfo(exe.wx_str(), unusedHandle, fiSize, fi.data()))
+        return "";
+    void *ver;
+    UINT sizeInfo;
+    if (!VerQueryValue(fi.data(), L"\\", &ver, &sizeInfo))
+        return "";
+    VS_FIXEDFILEINFO *info = (VS_FIXEDFILEINFO*)ver;
+    return wxString::Format("%d", LOWORD(info->dwFileVersionLS));
+#else
+    return "";
+#endif
+}
+
 bool PoeditApp::IsBetaVersion() const
 {
     wxString v = GetAppVersion();
@@ -124,9 +313,26 @@ extern void InitXmlResource();
 
 bool PoeditApp::OnInit()
 {
+    SetVendorName("Vaclav Slavik");
+    SetAppName("Poedit");
+
+#ifndef __WXOSX__
+    // create early for use in cmd line parsing
+    m_instanceChecker.reset(new wxSingleInstanceChecker);
+  #ifdef __UNIX__
+    m_instanceChecker->Create("poedit.lock", GetRuntimeDir());
+  #else
+    m_instanceChecker->CreateDefault();
+  #endif
+#endif
+
     if (!wxApp::OnInit())
         return false;
 
+#ifndef __WXOSX__
+    m_remoteServer.reset(new RemoteServer(this));
+#endif
+
     InitHiDPIHandling();
 
 #ifdef __WXOSX__
@@ -179,9 +385,6 @@ bool PoeditApp::OnInit()
     }
 #endif // __UNIX__
 
-    SetVendorName("Vaclav Slavik");
-    SetAppName("Poedit");
-
 #if defined(__WXOSX__)
     #define CFG_FILE (wxStandardPaths::Get().GetUserConfigDir() + "/net.poedit.Poedit.cfg")
 #elif defined(__UNIX__)
@@ -245,9 +448,11 @@ bool PoeditApp::OnInit()
     FileHistory().Load(*wxConfig::Get());
 #endif
 
+#ifndef DONT_MIGRATE_LEGACY_TM
     // NB: It's important to do this before TM is used for the first time.
     if ( !MigrateLegacyTranslationMemory() )
         return false;
+#endif
 
 #ifdef __WXMSW__
     AssociateFileTypeIfNeeded();
@@ -286,6 +491,9 @@ bool PoeditApp::OnInit()
     win_sparkle_set_appcast_url(appcast.utf8_str());
     win_sparkle_set_can_shutdown_callback(&PoeditApp::WinSparkle_CanShutdown);
     win_sparkle_set_shutdown_request_callback(&PoeditApp::WinSparkle_Shutdown);
+    auto buildnum = GetAppBuildNumber();
+    if (!buildnum.empty())
+        win_sparkle_set_app_build_version(buildnum.wc_str());
     win_sparkle_init();
 #endif
 
@@ -303,12 +511,26 @@ bool PoeditApp::OnInit()
 
 int PoeditApp::OnExit()
 {
+#ifndef __WXOSX__
+    m_instanceChecker.reset();
+    m_remoteServer.reset();
+#endif
+
+    // Keep any clipboard data available on Windows after the app terminates:
+    wxTheClipboard->Flush();
+
     // Make sure PoeditFrame instances schedules for deletion are deleted
     // early -- e.g. before wxConfig is destroyed, so they can save changes
     DeletePendingObjects();
 
     TranslationMemory::CleanUp();
 
+#ifdef HAVE_HTTP_CLIENT
+    CrowdinClient::CleanUp();
+#endif
+
+    concurrency_queue::cleanup();
+
 #ifdef USE_SPARKLE
     Sparkle_Cleanup();
 #endif // USE_SPARKLE
@@ -335,17 +557,38 @@ void PoeditApp::SetupLanguage()
     wxTranslations *trans = new wxTranslations();
     wxTranslations::Set(trans);
 
-#ifdef __WXGTK__
+    int language = wxLANGUAGE_DEFAULT;
+
+#if NEED_CHOOSELANG_UI
+    auto uilang = GetUILanguage();
+    if (!uilang.empty())
+    {
+        auto langinfo = wxLocale::FindLanguageInfo(uilang);
+        if (langinfo)
+        {
+            language = langinfo->Language;
+            uilang.clear(); // will go through locale
+        }
+    }
+#endif
+
     // Properly set locale is important for some aspects of GTK+ as well as
     // other things. It's also the common thing to do, so don't break
     // expectations needlessly:
-    m_locale.reset(new wxLocale());
-    m_locale->Init(wxLANGUAGE_DEFAULT, wxLOCALE_DONT_LOAD_DEFAULT);
-#endif
+    {
+        // suppress error logging because setting locale may fail and we want to
+        // handle that gracefully and invisibly:
+        wxLogNull null;
+        m_locale.reset(new wxLocale());
+        if (!m_locale->Init(language, wxLOCALE_DONT_LOAD_DEFAULT))
+            m_locale.reset();
 
 #if NEED_CHOOSELANG_UI
-    trans->SetLanguage(GetUILanguage());
+        if (!uilang.empty())
+            trans->SetLanguage(uilang);
 #endif
+    }
+
     trans->AddStdCatalog();
     trans->AddCatalog("poedit");
 
@@ -356,6 +599,10 @@ void PoeditApp::SetupLanguage()
 
     const wxLanguageInfo *info = wxLocale::FindLanguageInfo(bestTrans);
     g_layoutDirection = info ? info->LayoutDirection : wxLayout_Default;
+
+#ifdef _WXMSW__
+    win_sparkle_set_lang(bestTrans.utf8_str());
+#endif
 }
 
 wxLayoutDirection PoeditApp::GetLayoutDirection() const
@@ -366,7 +613,11 @@ wxLayoutDirection PoeditApp::GetLayoutDirection() const
 
 void PoeditApp::OpenNewFile()
 {
-    PoeditFrame::CreateWelcome();
+    PoeditFrame *unused = PoeditFrame::UnusedWindow(/*active=*/false);
+    if (unused)
+        unused->Raise();
+    else
+        PoeditFrame::CreateWelcome();
 }
 
 void PoeditApp::OpenFiles(const wxArrayString& names)
@@ -539,6 +790,7 @@ void PoeditApp::SetDefaultCfg(wxConfigBase *cfg)
 namespace
 {
 const char *CL_KEEP_TEMP_FILES = "keep-temp-files";
+const char *CL_HANDLE_POEDIT_URI = "handle-poedit-uri";
 }
 
 void PoeditApp::OnInitCmdLine(wxCmdLineParser& parser)
@@ -547,8 +799,9 @@ void PoeditApp::OnInitCmdLine(wxCmdLineParser& parser)
 
     parser.AddSwitch("", CL_KEEP_TEMP_FILES,
                      _("don't delete temporary files (for debugging)"));
-
-    parser.AddParam("catalog.po", wxCMD_LINE_VAL_STRING, 
+    parser.AddLongOption(CL_HANDLE_POEDIT_URI,
+                     _("handle a poedit:// URI"), wxCMD_LINE_VAL_STRING);
+    parser.AddParam("catalog.po", wxCMD_LINE_VAL_STRING,
                     wxCMD_LINE_PARAM_OPTIONAL | wxCMD_LINE_PARAM_MULTIPLE);
 }
 
@@ -560,6 +813,48 @@ bool PoeditApp::OnCmdLineParsed(wxCmdLineParser& parser)
     if ( parser.Found(CL_KEEP_TEMP_FILES) )
         TempDirectory::KeepFiles();
 
+#ifndef __WXOSX__
+    RemoteClient client(m_instanceChecker.get());
+    switch (client.ConnectIfNeeded())
+    {
+        case RemoteClient::Failure:
+        {
+            wxLogError(_("Failed to communicate with Poedit process."));
+            wxLog::FlushActive();
+            return false; // terminate program
+        }
+
+        case RemoteClient::Connected:
+        {
+            wxString poeditURI;
+            if (parser.Found(CL_HANDLE_POEDIT_URI, &poeditURI))
+                client.HandleCustomURI(poeditURI);
+
+            if (parser.GetParamCount() == 0)
+            {
+                client.Activate();
+            }
+            else
+            {
+                for (size_t i = 0; i < parser.GetParamCount(); i++)
+                    client.OpenFile(parser.GetParam(i));
+            }
+            return false; // terminate program
+        }
+
+        case RemoteClient::NoOtherInstance:
+            // fall through and handle normally
+            break;
+    }
+#endif
+
+    wxString poeditURI;
+    if (parser.Found(CL_HANDLE_POEDIT_URI, &poeditURI))
+    {
+        // handling the URI shows UI, so do it after OnInit() initialization:
+        CallAfter([=]{ HandleCustomURI(poeditURI); });
+    }
+
     for (size_t i = 0; i < parser.GetParamCount(); i++)
         gs_filesToOpen.Add(parser.GetParam(i));
 
@@ -567,6 +862,22 @@ bool PoeditApp::OnCmdLineParsed(wxCmdLineParser& parser)
 }
 
 
+void PoeditApp::HandleCustomURI(const wxString& uri)
+{
+    if (!uri.StartsWith("poedit://"))
+        return;
+
+#ifdef HAVE_HTTP_CLIENT
+    // nothing to do yet
+    if (CrowdinClient::Get().IsOAuthCallback(uri.ToStdString()))
+    {
+        CrowdinClient::Get().HandleOAuthCallback(uri.ToStdString());
+        return;
+    }
+#endif
+}
+
+
 // ---------------------------------------------------------------------------
 // exceptions handling
 // ---------------------------------------------------------------------------
@@ -584,7 +895,7 @@ bool PoeditApp::OnExceptionInMainLoop()
 #ifdef __WXOSX__
     catch ( NSException *e )
     {
-        wxLogError(_("Unhandled exception occurred: %s"), wxStringFromNS([e reason]));
+        wxLogError(_("Unhandled exception occurred: %s"), str::to_wx([e reason]));
     }
 #endif
     catch ( ... )
@@ -606,6 +917,9 @@ BEGIN_EVENT_TABLE(PoeditApp, wxApp)
    EVT_MENU           (wxID_NEW,                  PoeditApp::OnNew)
    EVT_MENU           (XRCID("menu_new_from_pot"),PoeditApp::OnNew)
    EVT_MENU           (wxID_OPEN,                 PoeditApp::OnOpen)
+ #ifdef HAVE_HTTP_CLIENT
+   EVT_MENU           (XRCID("menu_open_crowdin"),PoeditApp::OnOpenFromCrowdin)
+ #endif
  #ifndef __WXOSX__
    EVT_MENU_RANGE     (wxID_FILE1, wxID_FILE9,    PoeditApp::OnOpenHist)
  #endif
@@ -657,11 +971,10 @@ void PoeditApp::OnOpen(wxCommandEvent&)
     wxString path = wxConfig::Get()->Read("last_file_path", wxEmptyString);
 
     wxFileDialog dlg(nullptr,
-                     _("Open catalog"),
+                     OSX_OR_OTHER("", _("Open catalog")),
                      path,
                      wxEmptyString,
-                     wxString::Format("%s (*.po)|*.po|%s (*.*)|*.*",
-                         _("PO Translation Files"), _("All Files")),
+                     Catalog::GetAllTypesFileMask(),
                      wxFD_OPEN | wxFD_FILE_MUST_EXIST | wxFD_MULTIPLE);
 
     if (dlg.ShowModal() == wxID_OK)
@@ -681,6 +994,17 @@ void PoeditApp::OnOpen(wxCommandEvent&)
 }
 
 
+#ifdef HAVE_HTTP_CLIENT
+void PoeditApp::OnOpenFromCrowdin(wxCommandEvent& event)
+{
+    TRY_FORWARD_TO_ACTIVE_WINDOW( OnOpenFromCrowdin(event) );
+
+    PoeditFrame *f = PoeditFrame::CreateEmpty();
+    f->OnOpenFromCrowdin(event);
+}
+#endif
+
+
 #ifndef __WXOSX__
 void PoeditApp::OnOpenHist(wxCommandEvent& event)
 {
@@ -798,7 +1122,7 @@ void PoeditApp::OpenPoeditWeb(const wxString& path)
 
 static NSMenuItem *AddNativeItem(NSMenu *menu, int pos, const wxString&text, SEL ac, NSString *key)
 {
-    NSString *str = wxStringToNS(text);
+    NSString *str = str::to_NS(text);
     if (pos == -1)
         return [menu addItemWithTitle:str action:ac keyEquivalent:key];
     else
@@ -950,7 +1274,7 @@ void PoeditApp::InstallOpenRecentMenu(wxMenuBar *bar)
         return;
 
     NSMenu *native = fileMenu->GetHMenu();
-    NSMenuItem *nativeItem = [native itemWithTitle:wxStringToNS(item->GetItemLabelText())];
+    NSMenuItem *nativeItem = [native itemWithTitle:str::to_NS(item->GetItemLabelText())];
     if (!nativeItem)
         return;
 
