@@ -33,6 +33,7 @@
 #include <wx/checkbox.h>
 #include <wx/choice.h>
 #include <wx/checklst.h>
+#include <wx/notebook.h>
 #include <wx/sizer.h>
 #include <wx/stattext.h>
 #include <wx/fontutil.h>
@@ -52,6 +53,7 @@
 #include "edapp.h"
 #include "edframe.h"
 #include "catalog.h"
+#include "crowdin_gui.h"
 #include "hidpi.h"
 #include "tm/transmem.h"
 #include "chooselang.h"
@@ -80,7 +82,7 @@ class PrefsPanel : public wxPanel
 {
 public:
     PrefsPanel(wxWindow *parent)
-        : wxPanel(parent), m_supressDataTransfer(0)
+        : wxPanel(parent), m_suppressDataTransfer(0)
     {
 #ifdef __WXOSX__
         // Refresh the content of prefs panels when re-opening it.
@@ -100,14 +102,14 @@ public:
 
     bool TransferDataToWindow() override
     {
-        if (m_supressDataTransfer)
+        if (m_suppressDataTransfer)
             return false;
-        m_supressDataTransfer++;
+        m_suppressDataTransfer++;
         InitValues(*wxConfig::Get());
-        m_supressDataTransfer--;
+        m_suppressDataTransfer--;
 
         // This is a "bit" of a hack: we take advantage of being in the last point before
-        // showing the window and re-layout it on the off chance that some data transfered
+        // showing the window and re-layout it on the off chance that some data transferred
         // into the window affected its size. And, currently more importantly, to reflect
         // ExplanationLabel instances' rewrapping.
         Fit();
@@ -119,11 +121,11 @@ public:
 
     bool TransferDataFromWindow() override
     {
-        if (m_supressDataTransfer)
+        if (m_suppressDataTransfer)
             return false;
-        m_supressDataTransfer++;
+        m_suppressDataTransfer++;
         SaveValues(*wxConfig::Get());
-        m_supressDataTransfer--;
+        m_suppressDataTransfer--;
         return true;
     }
 
@@ -137,18 +139,10 @@ protected:
     virtual void InitValues(const wxConfigBase& cfg) = 0;
     virtual void SaveValues(wxConfigBase& cfg) = 0;
 
-    int m_supressDataTransfer;
+    int m_suppressDataTransfer;
 };
 
 
-#ifdef __WXOSX__
-    #define BORDER_WIN(dir, n) Border(dir, 0)
-    #define BORDER_OSX(dir, n) Border(dir, n)
-#else
-    #define BORDER_WIN(dir, n) Border(dir, n)
-    #define BORDER_OSX(dir, n) Border(dir, 0)
-#endif
-
 
 class GeneralPageWindow : public PrefsPanel
 {
@@ -286,7 +280,7 @@ public:
         m_useFontText->SetValue(cfg.ReadBool("custom_font_text_use", false));
 
         #if defined(__WXOSX__)
-            #define DEFAULT_FONT "Helvetica"
+            #define DEFAULT_FONT "Helvetica Neue"
         #elif defined(__WXMSW__)
             #define DEFAULT_FONT "Arial"
         #elif defined(__WXGTK__)
@@ -473,8 +467,7 @@ private:
             _("Select translation files to import"),
             wxEmptyString,
             wxEmptyString,
-			wxString::Format("%s (*.po)|*.po|%s (*.*)|*.*",
-                _("PO Translation Files"), _("All Files")),
+			Catalog::GetTypesFileMask({Catalog::Type::PO}),
 			wxFD_OPEN | wxFD_FILE_MUST_EXIST | wxFD_MULTIPLE));
 
         dlg->ShowWindowModalThenDo([=](int retcode){
@@ -489,15 +482,15 @@ private:
                                       (int)paths.size() * 2 + 1,
                                       this,
                                       wxPD_APP_MODAL|wxPD_AUTO_HIDE|wxPD_CAN_ABORT);
-            auto tm = TranslationMemory::Get().CreateWriter();
+            auto tm = TranslationMemory::Get().GetWriter();
             int step = 0;
             for (size_t i = 0; i < paths.size(); i++)
             {
-                std::unique_ptr<Catalog> cat(new Catalog(paths[i]));
+                CatalogPtr cat = std::make_shared<Catalog>(paths[i]);
                 if (!progress.Update(++step))
                     break;
                 if (cat->IsOk())
-                    tm->Insert(*cat);
+                    tm->Insert(cat);
                 if (!progress.Update(++step))
                     break;
             }
@@ -520,7 +513,7 @@ private:
         dlg->ShowWindowModalThenDo([this,dlg](int retcode){
             if (retcode == wxID_YES) {
                 wxBusyCursor bcur;
-                auto tm = TranslationMemory::Get().CreateWriter();
+                auto tm = TranslationMemory::Get().GetWriter();
                 tm->DeleteAll();
                 tm->Commit();
                 UpdateStats();
@@ -542,7 +535,7 @@ class TMPage : public wxPreferencesPage
 public:
     wxString GetName() const override
     {
-#ifdef __WXOSX__
+#if defined(__WXOSX__) || defined(__WXGTK__)
         // TRANSLATORS: This is abbreviation of "Translation Memory" used in Preferences on OS X.
         // Long text looks weird there, too short (like TM) too, but less so. "General" is about ideal
         // length there.
@@ -626,7 +619,7 @@ public:
     }
 
 private:
-    /// Called to launch dialog for editting parser properties.
+    /// Called to launch dialog for editing parser properties.
     template<typename TFunctor>
     void EditExtractor(int num, TFunctor completionHandler)
     {
@@ -663,19 +656,19 @@ private:
             wxID_OK
         );
 
-        m_supressDataTransfer++;
+        m_suppressDataTransfer++;
         dlg->ShowWindowModalThenDo([=](int retcode){
-            m_supressDataTransfer--;
+            m_suppressDataTransfer--;
             (void)dlg; // force use
             if (retcode == wxID_OK)
             {
                 Extractor& nfo = m_extractors.Data[num];
-                nfo.Name = extractor_language->GetValue();
-                nfo.Extensions = extractor_extensions->GetValue();
-                nfo.Command = extractor_command->GetValue();
-                nfo.KeywordItem = extractor_keywords->GetValue();
-                nfo.FileItem = extractor_files->GetValue();
-                nfo.CharsetItem = extractor_charset->GetValue();
+                nfo.Name = extractor_language->GetValue().Strip(wxString::both);
+                nfo.Extensions = extractor_extensions->GetValue().Strip(wxString::both);
+                nfo.Command = extractor_command->GetValue().Strip(wxString::both);
+                nfo.KeywordItem = extractor_keywords->GetValue().Strip(wxString::both);
+                nfo.FileItem = extractor_files->GetValue().Strip(wxString::both);
+                nfo.CharsetItem = extractor_charset->GetValue().Strip(wxString::both);
                 m_list->SetString(num, nfo.Name);
             }
             completionHandler(retcode == wxID_OK);
@@ -684,7 +677,7 @@ private:
 
     void OnNewExtractor(wxCommandEvent&)
     {
-        m_supressDataTransfer++;
+        m_suppressDataTransfer++;
 
         Extractor info;
         m_extractors.Data.push_back(info);
@@ -702,7 +695,7 @@ private:
                 m_extractors.Data.erase(m_extractors.Data.begin() + index);
             }
 
-            m_supressDataTransfer--;
+            m_suppressDataTransfer--;
 
             if (wxPreferencesEditor::ShouldApplyChangesImmediately())
                 TransferDataFromWindow();
@@ -720,11 +713,26 @@ private:
     void OnDeleteExtractor(wxCommandEvent&)
     {
         int index = m_list->GetSelection();
-        m_extractors.Data.erase(m_extractors.Data.begin() + index);
-        m_list->Delete(index);
 
-        if (wxPreferencesEditor::ShouldApplyChangesImmediately())
-            TransferDataFromWindow();
+        auto title = _("Delete extractor");
+        auto main = wxString::Format(_(L"Are you sure you want to delete the “%s” extractor?"), m_extractors.Data[index].Name);
+
+        wxWindowPtr<wxMessageDialog> dlg(new wxMessageDialog(this, main, title, wxYES_NO | wxNO_DEFAULT | wxICON_QUESTION));
+        #ifdef __WXOSX__
+        dlg->SetExtendedMessage(" "); // prevent wx from using the title stupidly
+        #endif
+        dlg->SetYesNoLabels(_("Delete"), _("Cancel"));
+
+        dlg->ShowWindowModalThenDo([this,index,dlg](int retcode){
+            if (retcode == wxID_YES)
+            {
+                m_extractors.Data.erase(m_extractors.Data.begin() + index);
+                m_list->Delete(index);
+
+                if (wxPreferencesEditor::ShouldApplyChangesImmediately())
+                    TransferDataFromWindow();
+            }
+        });
     }
 
     void OnEnableExtractor(wxCommandEvent& e)
@@ -752,6 +760,58 @@ public:
 
 
 
+#ifdef HAVE_HTTP_CLIENT
+class AccountsPageWindow : public PrefsPanel
+{
+public:
+    AccountsPageWindow(wxWindow *parent) : PrefsPanel(parent)
+    {
+        wxSizer *topsizer = new wxBoxSizer(wxVERTICAL);
+        m_login = new CrowdinLoginPanel(this);
+        topsizer->Add(m_login, wxSizerFlags(1).Expand().Border(wxALL, PX(15)));
+        SetSizer(topsizer);
+
+    #ifdef __WXOSX__
+        // This window was created on demand, init immediately:
+        m_login->EnsureInitialized();
+    #else
+        // On other platforms, notebook pages are all created at once. Don't do
+        // the expensive initialization until shown for the first time. This code
+        // is a hack that takes advantage of wxPreferencesEditor's implementation
+        // detail, but oh well:
+        auto notebook = dynamic_cast<wxNotebook*>(parent);
+        if (notebook)
+        {
+            notebook->Bind(wxEVT_NOTEBOOK_PAGE_CHANGED, [=](wxBookCtrlEvent& e){
+                e.Skip();
+                if (notebook->GetPage(e.GetSelection()) == this)
+                    m_login->EnsureInitialized();
+            });
+        }
+    #endif
+    }
+
+    void InitValues(const wxConfigBase&) override
+    {
+    }
+
+    void SaveValues(wxConfigBase&) override
+    {
+    }
+
+private:
+    CrowdinLoginPanel *m_login;
+};
+
+class AccountsPage : public wxPreferencesPage
+{
+public:
+    wxString GetName() const override { return PXNotebookTab(_("Accounts")); }
+    wxBitmap GetLargeIcon() const override { return wxArtProvider::GetBitmap("Prefs-Accounts"); }
+    wxWindow *CreateWindow(wxWindow *parent) override { return new AccountsPageWindow(parent); }
+};
+#endif // HAVE_HTTP_CLIENT
+
 
 #ifdef HAS_UPDATES_CHECK
 class UpdatesPageWindow : public PrefsPanel
@@ -918,6 +978,9 @@ std::unique_ptr<PoeditPreferencesEditor> PoeditPreferencesEditor::Create()
     p->AddPage(new GeneralPage);
     p->AddPage(new TMPage);
     p->AddPage(new ExtractorsPage);
+#ifdef HAVE_HTTP_CLIENT
+    p->AddPage(new AccountsPage);
+#endif
 #ifdef HAS_UPDATES_CHECK
     p->AddPage(new UpdatesPage);
 #endif
