@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 2000-2014 Vaclav Slavik
+ *  Copyright (C) 2000-2015 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -61,7 +61,7 @@ ProgressInfo::ProgressInfo(wxWindow *parent, const wxString& title)
 {
     m_cancelled = false;
     m_dlg = new ProgressDlg(&m_cancelled);
-    wxXmlResource::Get()->LoadDialog(m_dlg, parent, "parser_progress");
+    wxXmlResource::Get()->LoadDialog(m_dlg, parent, "extractor_progress");
     m_dlg->SetTitle(title);
     m_dlg->Show(true);
     m_disabler = new wxWindowDisabler(m_dlg);
@@ -78,7 +78,7 @@ void ProgressInfo::SetGaugeMax(int limit)
     XRCCTRL(*m_dlg, "progress", wxGauge)->SetRange(limit);
 }
 
-void ProgressInfo::UpdateGauge(int increment)
+bool ProgressInfo::UpdateGauge(int increment)
 {
     wxGauge *g = XRCCTRL(*m_dlg, "progress", wxGauge);
     g->SetValue(g->GetValue() + increment);
@@ -89,6 +89,8 @@ void ProgressInfo::UpdateGauge(int increment)
     txt->SetLabel(txt->GetLabel());
     txt->Update();
 #endif
+
+    return !m_cancelled;
 }
 
 void ProgressInfo::ResetGauge(int value)
