@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 2014 Vaclav Slavik
+ *  Copyright (C) 2014-2015 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -25,7 +25,7 @@
 
 #include "spellchecking.h"
 
-#if USE_SPELLCHECKING
+#include "str_helpers.h"
 
 #ifdef __WXGTK__
     #include <gtk/gtk.h>
@@ -34,10 +34,6 @@
     }
 #endif
 
-#ifdef __WXOSX__
-    #include <wx/cocoa/string.h>
-#endif
-
 #ifdef __WXMSW__
     #include <wx/msw/wrapwin.h>
     #include <Richedit.h>
@@ -138,7 +134,7 @@ bool InitTextCtrlSpellchecker(wxTextCtrl *text, bool enable, const Language& lan
 #ifdef __WXOSX__
 bool SetSpellcheckerLang(const wxString& lang)
 {
-    NSString *nslang = wxNSStringWithWxString(lang);
+    NSString *nslang = str::to_NS(lang);
     NSSpellChecker *sc = [NSSpellChecker sharedSpellChecker];
     [sc setAutomaticallyIdentifiesLanguages:NO];
     return [sc setLanguage: nslang];
@@ -193,5 +189,3 @@ void ShowSpellcheckerHelp()
     wxGetApp().OpenPoeditWeb("/trac/wiki/Doc/" SPELL_HELP_PAGE);
 }
 #endif // !__WXMSW__
-
-#endif // USE_SPELLCHECKING
\ No newline at end of file
