@@ -28,6 +28,7 @@
 
 #include <wx/textctrl.h>
 #include <memory>
+#include <vector>
 
 #include "syntaxhighlighter.h"
 
@@ -38,6 +39,8 @@
     - Allow setting text programatically, without user-input processing (OS X)
     - Disable user-usable rich text support
     - Stylistic tweaks (padding and such)
+    - Generic undo/redo implementation for GTK
+    - Search highlighting
  */
 class CustomizedTextCtrl : public wxTextCtrl
 {
@@ -46,18 +49,50 @@ public:
 
     CustomizedTextCtrl(wxWindow *parent, wxWindowID winid, long style = 0);
 
+    /// Show find result indicator at given part of the text
+    void ShowFindIndicator(int from, int length);
+
+#ifdef __WXGTK__
+    // Undo/redo implementation:
+    void BeginUndoGrouping();
+    void EndUndoGrouping();
+    void SaveSnapshot();
+#endif
+
 protected:
 #ifdef __WXOSX__
     void DoSetValue(const wxString& value, int flags) override;
     wxString DoGetValue() const override;
 #endif
 
-#ifdef __WXMSW__
+#if defined(__WXMSW__) || defined(__WXGTK__)
     bool DoCopy();
     void OnCopy(wxClipboardTextEvent& event);
     void OnCut(wxClipboardTextEvent& event);
     void OnPaste(wxClipboardTextEvent& event);
-#endif // __WXMSW__
+
+    virtual wxString DoCopyText(long from, long to);
+    virtual void DoPasteText(long from, long to, const wxString& s);
+#endif // __WXMSW__/__WXGTK__
+
+#ifdef __WXGTK__
+    struct Snapshot
+    {
+        wxString text;
+        long insertionPoint;
+    };
+
+    void DoSetValue(const wxString& value, int flags) override;
+    void OnText(wxCommandEvent& event);
+    bool CanUndo() const override;
+    bool CanRedo() const override;
+    void Undo() override;
+    void Redo() override;
+
+    std::vector<Snapshot> m_history;
+    size_t m_historyIndex; // where in the vector to insert the next snapshot
+    int m_historyLocks;
+#endif // __WXGTK__
 };
 
 
@@ -69,7 +104,23 @@ public:
 
     void SetLanguageRTL(bool isRTL);
 
+    // Set and get control's text as plain/raw text, with no escaping or formatting.
+    // This is the "true" representation, with e.g newlines included. The version
+    // displayed to the user includes syntax highlighting and escaping of some characters
+    // (e.g. tabs shown as \t, newlines as \n followed by newline).
+    void SetPlainText(const wxString& s) { SetValue(EscapePlainText(s)); }
+    wxString GetPlainText() const { return UnescapePlainText(GetValue()); }
+
+    // Apply escaping as described in SetPlainText:
+    static wxString EscapePlainText(const wxString& s);
+    static wxString UnescapePlainText(const wxString& s);
+
 protected:
+#if defined(__WXMSW__) || defined(__WXGTK__)
+    wxString DoCopyText(long from, long to) override;
+    void DoPasteText(long from, long to, const wxString& s) override;
+#endif
+
 #ifdef __WXMSW__
     void DoSetValue(const wxString& value, int flags) override;
     void UpdateRTLStyle();
@@ -81,7 +132,7 @@ protected:
     void DoSetValue(const wxString& value, int flags) override;
 #endif
 
-private:
+protected:
     void HighlightText();
 
     class Attributes;
@@ -105,12 +156,17 @@ public:
     TranslationTextCtrl(wxWindow *parent, wxWindowID winid);
 
     /// Sets the value to something the user wrote
-    void SetValueUserWritten(const wxString& value);
+    void SetPlainTextUserWritten(const wxString& value);
 
 protected:
+    void OnKeyDown(wxKeyEvent& e);
+    void OnText(wxCommandEvent& e);
+
 #ifdef __WXOSX__
     void DoSetValue(const wxString& value, int flags) override;
 #endif
+
+    bool m_lastKeyWasReturn;
 };
 
 #endif // Poedit_text_control_h
