@@ -29,12 +29,17 @@
 #include <wx/filename.h>
 #include <wx/log.h>
 #include <wx/config.h>
-#include <wx/display.h>
+
+#if wxUSE_GUI
+    #include <wx/display.h>
+#endif
 
 #ifdef __WXOSX__
-#include "osx_helpers.h"
+    #include <Foundation/Foundation.h>
 #endif
 
+#include "str_helpers.h"
+
 wxString EscapeMarkup(const wxString& str)
 {
     wxString s(str);
@@ -44,6 +49,43 @@ wxString EscapeMarkup(const wxString& str)
     return s;
 }
 
+
+wxFileName CommonDirectory(const wxFileName& a, const wxFileName& b)
+{
+    if (!a.IsOk())
+        return wxFileName::DirName(b.GetPath());;
+    if (!b.IsOk())
+        return wxFileName::DirName(a.GetPath());;
+
+    const auto& dirs_a = a.GetDirs();
+    const auto& dirs_b = b.GetDirs();
+
+    wxFileName d = wxFileName::DirName(a.GetPath());
+    const auto count = std::min(dirs_a.size(), dirs_b.size());
+    size_t i = 0;
+    for (i = 0; i < count; i++)
+    {
+        if (dirs_a[i] != dirs_b[i])
+            break;
+    }
+    while (d.GetDirCount() != i)
+        d.RemoveLastDir();
+    return d;
+}
+
+wxFileName MakeFileName(const wxString& path)
+{
+    wxFileName fn;
+    if (path.empty())
+        return fn;
+    if (wxFileName::DirExists(path) || path.Last() == wxFILE_SEP_PATH)
+        fn.AssignDir(path);
+    else
+        fn.Assign(path);
+    fn.Normalize();
+    return fn;
+}
+
 // ----------------------------------------------------------------------
 // TempDirectory
 // ----------------------------------------------------------------------
@@ -96,6 +138,14 @@ TempDirectory::~TempDirectory()
     if ( m_dir.empty() )
         return;
 
+    Clear();
+
+    wxLogTrace("poedit.tmp", "removing temp dir %s", m_dir.c_str());
+    wxFileName::Rmdir(m_dir);
+}
+
+void TempDirectory::Clear()
+{
     if ( ms_keepFiles )
     {
         wxLogTrace("poedit.tmp", "keeping temp files in %s", m_dir.c_str());
@@ -110,12 +160,8 @@ TempDirectory::~TempDirectory()
             wxRemoveFile(*i);
         }
     }
-
-    wxLogTrace("poedit.tmp", "removing temp dir %s", m_dir.c_str());
-    wxFileName::Rmdir(m_dir);
 }
 
-
 wxString TempDirectory::CreateFileName(const wxString& suffix)
 {
     wxString s = wxString::Format("%s%c%d%s",
@@ -132,13 +178,17 @@ wxString TempDirectory::CreateFileName(const wxString& suffix)
 // TempOutputFile
 // ----------------------------------------------------------------------
 
-TempOutputFileFor::TempOutputFileFor(const wxString& filename)
+TempOutputFileFor::TempOutputFileFor(const wxString& filename) : m_filenameFinal(filename)
 {
     wxString path, name, ext;
     wxFileName::SplitPath(filename, &path, &name, &ext);
+    if (path.empty())
+        path = ".";
+    if (!ext.empty())
+        ext = "." + ext;
 
 #ifdef __WXOSX__
-    NSURL *fileUrl = [NSURL fileURLWithPath:wxStringToNS(filename)];
+    NSURL *fileUrl = [NSURL fileURLWithPath:str::to_NS(filename)];
     NSURL *tempdirUrl =
         [[NSFileManager defaultManager] URLForDirectory:NSItemReplacementDirectory
                                                inDomain:NSUserDomainMask
@@ -146,38 +196,92 @@ TempOutputFileFor::TempOutputFileFor(const wxString& filename)
                                                  create:YES
                                                   error:nil];
     if (tempdirUrl)
+        m_tempDir = str::to_wx([tempdirUrl path]);
+#endif
+
+    wxString counter;
+    for (;;)
     {
-        NSURL *newFileUrl = [tempdirUrl URLByAppendingPathComponent:[fileUrl lastPathComponent]];
-        NSString *newFilePath = [newFileUrl path];
-        m_filenameTmp = wxStringFromNS(newFilePath);
-        m_tempDir = wxStringFromNS([tempdirUrl path]);
-    }
-    // else: fall through to the generic code
+#ifdef __WXOSX__
+        if (!m_tempDir.empty())
+        {
+            m_filenameTmp = m_tempDir + wxFILE_SEP_PATH + name + counter + ext;
+        }
+        else
 #endif // __WXOSX__
+        {
+            // Temp filenames may be ugly, nobody cares. Make them safe for
+            // Unicode-unfriendly uses on Windows, i.e. 8.3 without non-ASCII
+            // characters:
+            auto base = CliSafeFileName(path) + wxFILE_SEP_PATH;
+#ifdef __WXMSW__
+            // this is OK, ToAscii() replaces non-ASCII with '_':
+            base += name.ToAscii();
+#else
+            base += name;
+#endif
+            m_filenameTmp = base + ".temp" + counter + ext;
+        }
 
-    if (m_filenameTmp.empty())
-    {
-        m_filenameTmp = filename + ".temp." + ext;
+        if (!wxFileExists(m_filenameTmp))
+            break; // good!
+
+        counter += wchar_t('a' + rand() % 26);
     }
-
-    if ( wxFileExists(m_filenameTmp) )
-        wxRemoveFile(m_filenameTmp);
 }
 
+bool TempOutputFileFor::Commit()
+{
+    return wxRenameFile(m_filenameTmp, m_filenameFinal, /*overwrite=*/true);
+}
 
 TempOutputFileFor::~TempOutputFileFor()
 {
 #ifdef __WXOSX__
     if (!m_tempDir.empty())
         wxFileName::Rmdir(m_tempDir, wxPATH_RMDIR_RECURSIVE);
+#else
+    if (wxFileExists(m_filenameTmp))
+        wxRemoveFile(m_filenameTmp);
 #endif
 }
 
+#ifdef __WXMSW__
+wxString CliSafeFileName(const wxString& fn)
+{
+    if (fn.IsAscii())
+    {
+        return fn;
+    }
+    else if (wxFileExists(fn) || wxDirExists(fn))
+    {
+        return wxFileName(fn).GetShortPath();
+    }
+    else
+    {
+        wxString path, name, ext;
+        wxFileName::SplitPath(fn, &path, &name, &ext);
+        if (path.empty())
+            path = ".";
+        if (wxDirExists(path))
+        {
+            auto p = wxFileName(path).GetShortPath() + wxFILE_SEP_PATH + name;
+            if (!ext.empty())
+                p += "." + ext;
+            return p;
+        }
+    }
+    return fn;
+}
+#endif // __WXMSW__
+
 
 // ----------------------------------------------------------------------
 // Helpers for persisting windows' state
 // ----------------------------------------------------------------------
 
+#if wxUSE_GUI
+
 void SaveWindowState(const wxTopLevelWindow *win, int flags)
 {
 #ifdef __WXOSX__
@@ -226,7 +330,14 @@ void RestoreWindowState(wxTopLevelWindow *win, const wxSize& defaultSize, int fl
         int width = (int)cfg->Read(path + "w", defaultSize.x);
         int height = (int)cfg->Read(path + "h", defaultSize.y);
         if ( width != -1 || height != -1 )
+        {
+            // filter out ridiculous sizes:
+            if (width != -1 && width < 100)
+                width = defaultSize.x;
+            if (height != -1 && height < 100)
+                height = defaultSize.y;
             win->SetClientSize(width, height);
+        }
     }
 
 #if defined(__WXMSW__) || defined(__WXOSX__)
@@ -263,17 +374,10 @@ void RestoreWindowState(wxTopLevelWindow *win, const wxSize& defaultSize, int fl
 
     // If the window is completely out of all screens (e.g. because
     // screens configuration changed), move it to primary screen:
-#ifdef __WXOSX__
-    if ( win->GetPosition().x < 0 || win->GetPosition().y < 0 )
-        win->Move(20, 30);
-#else
     if ( wxDisplay::GetFromWindow(win) == wxNOT_FOUND )
-        win->Move(0, 0);
-#endif
-
+        win->Move(20, 40);
 #endif // __WXMSW__/__WXOSX__
 
-#ifndef __WXOSX__
     // If the window is larger than current screen, resize it to fit:
     int display = wxDisplay::GetFromWindow(win);
     if ( display == wxNOT_FOUND )
@@ -283,10 +387,13 @@ void RestoreWindowState(wxTopLevelWindow *win, const wxSize& defaultSize, int fl
 
     wxRect winRect = win->GetRect();
     if ( winRect.GetPosition() == wxDefaultPosition )
-        winRect.SetPosition(screenRect.GetPosition()); // not place yet, fake it
+        winRect.SetPosition(screenRect.GetPosition()); // not placed yet, fake it
 
     if ( !screenRect.Contains(winRect) )
     {
+        // Don't crop the window immediately, because it could become too small
+        // due to it. Try to move it to the center of the screen first, then crop.
+        winRect = winRect.CenterIn(screenRect);
         winRect.Intersect(screenRect);
         win->SetSize(winRect);
     }
@@ -296,5 +403,6 @@ void RestoreWindowState(wxTopLevelWindow *win, const wxSize& defaultSize, int fl
     {
         win->Maximize();
     }
-#endif // !__WXOSX__
 }
+
+#endif // wxUSE_GUI
