@@ -1,7 +1,7 @@
 ﻿/*
  *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 2013-2014 Vaclav Slavik
+ *  Copyright (C) 2013-2015 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -25,8 +25,11 @@
 
 #include "welcomescreen.h"
 
+#include "crowdin_gui.h"
 #include "edapp.h"
 #include "edframe.h"
+#include "hidpi.h"
+#include "str_helpers.h"
 
 #include <wx/dcbuffer.h>
 #include <wx/statbmp.h>
@@ -39,9 +42,7 @@
 #include <wx/hyperlink.h>
 #include <wx/xrc/xmlres.h>
 
-#ifdef __WXOSX__
-#include <wx/cocoa/string.h>
-#else
+#ifndef __WXOSX__
 #include <wx/commandlinkbutton.h>
 #endif
 
@@ -50,29 +51,65 @@ namespace
 
 #ifdef __WXOSX__
 
-class ActionButton : public wxButton
+class ActionButton : public wxWindow
 {
 public:
     ActionButton(wxWindow *parent, wxWindowID winid, const wxString& label, const wxString& note)
-        : wxButton(parent, winid, "", wxDefaultPosition, wxSize(500, 50))
+        : wxWindow(parent, wxID_ANY, wxDefaultPosition, wxSize(500, 65))
     {
-        NSButton *btn = (NSButton*)GetHandle();
+        m_button = new wxButton(this, winid, "");
+        auto sizer = new wxBoxSizer(wxHORIZONTAL);
+        sizer->Add(m_button, wxSizerFlags(1).Expand());
+        SetSizer(sizer);
+
+        NSButton *btn = (NSButton*)m_button->GetHandle();
+
+        NSString *str = str::to_NS(label + "\n" + note);
+        NSRange topLine = NSMakeRange(0, label.length() + 1);
+        NSRange bottomLine = NSMakeRange(label.length() + 1, note.length());
 
-        NSString *str = wxNSStringWithWxString(label + "\n" + note);
         NSMutableAttributedString *s = [[NSMutableAttributedString alloc] initWithString:str];
         [s addAttribute:NSFontAttributeName
-                  value:[NSFont boldSystemFontOfSize:0]
-                  range:NSMakeRange(0, label.length())];
+                  value:[NSFont systemFontOfSize:[NSFont systemFontSize]+1]
+                  range:topLine];
         [s addAttribute:NSFontAttributeName
-                  value:[NSFont systemFontOfSize:[NSFont systemFontSize]-1.5]
-                  range:NSMakeRange(label.length()+1, note.length())];
+                  value:[NSFont systemFontOfSize:[NSFont systemFontSizeForControlSize:NSSmallControlSize]]
+                  range:bottomLine];
+        [s addAttribute:NSForegroundColorAttributeName
+                  value:[NSColor grayColor]
+                  range:bottomLine];
+
+        NSMutableParagraphStyle *paragraphStyle = [[NSMutableParagraphStyle alloc] init];
+        paragraphStyle.headIndent = 10.0;
+        paragraphStyle.firstLineHeadIndent = 10.0;
+        paragraphStyle.tailIndent = -10.0;
+        paragraphStyle.lineSpacing = 2.0;
+        [s addAttribute:NSParagraphStyleAttributeName
+                  value:paragraphStyle
+                  range:NSMakeRange(0, [s length])];
+
         [btn setAttributedTitle:s];
         [btn setShowsBorderOnlyWhileMouseInside:YES];
         [btn setBezelStyle:NSSmallSquareBezelStyle];
         [btn setButtonType:NSMomentaryPushInButton];
 
-        SetBackgroundColour(wxColour("#e8fcdb"));
+        SetBackgroundColour(wxColour("#F2FCE2"));
+        Bind(wxEVT_PAINT, &ActionButton::OnPaint, this);
     }
+
+private:
+    void OnPaint(wxPaintEvent&)
+    {
+        wxPaintDC dc(this);
+        wxRect rect(dc.GetSize());
+        auto bg = GetBackgroundColour();
+        dc.SetBrush(bg);
+        dc.SetPen(bg.ChangeLightness(90));
+        dc.DrawRoundedRectangle(rect, 2);
+        dc.DrawRectangle(rect.x+1, rect.y+rect.height-2, rect.width-2, 2);
+    }
+
+    wxButton *m_button;
 };
 
 #elif defined(__WXGTK__)
@@ -121,25 +158,20 @@ typedef wxStaticText HeaderStaticText;
 
 WelcomeScreenBase::WelcomeScreenBase(wxWindow *parent)
     : wxPanel(parent, wxID_ANY),
-      m_clrHeader("#444444"),
+      m_clrHeader("#555555"),
       m_clrNorm("#444444"),
       m_clrSub("#aaaaaa")
 {
-#ifdef  __WXMSW__
     SetBackgroundColour(wxColour("#fffcf5"));
-#else
-    SetBackgroundStyle(wxBG_STYLE_PAINT);
-    Bind(wxEVT_PAINT, &WelcomeScreenPanel::OnPaint, this);
-#endif
 
-#if defined(__WXMAC__)
-    #define HEADER_FACE "Helvetica Neue"
-    m_fntHeader = wxFont(wxFontInfo(30).FaceName(HEADER_FACE).Light());
-    m_fntNorm = wxFont(wxFontInfo(13).FaceName(HEADER_FACE).Light());
-    m_fntSub = wxFont(wxFontInfo(11).FaceName(HEADER_FACE).Light());
+#if defined(__WXOSX__)
+    auto guiface = wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT).GetFaceName();
+    m_fntHeader = wxFont(wxFontInfo(30).FaceName(guiface).Light());
+    m_fntNorm = wxFont(wxFontInfo(13).FaceName(guiface).Light());
+    m_fntSub = wxFont(wxFontInfo(11).FaceName(guiface).Light());
 #elif defined(__WXMSW__)
     #define HEADER_FACE "Segoe UI"
-    m_fntHeader = wxFont(wxFontInfo(24).FaceName("Segoe UI Light"));
+    m_fntHeader = wxFont(wxFontInfo(20).FaceName("Segoe UI Light").AntiAliased());
     m_fntNorm = wxFont(wxFontInfo(10).FaceName(HEADER_FACE));
     m_fntSub = wxFont(wxFontInfo(9).FaceName(HEADER_FACE));
 #else
@@ -158,24 +190,13 @@ WelcomeScreenBase::WelcomeScreenBase(wxWindow *parent)
 }
 
 
-void WelcomeScreenBase::OnPaint(wxPaintEvent&)
-{
-    wxAutoBufferedPaintDC dc(this);
-    wxRect rect(dc.GetSize());
-
-    dc.GradientFillLinear(rect, wxColour("#ffffff"), wxColour("#fffceb"), wxBOTTOM);
-}
-
-
-
-
 WelcomeScreenPanel::WelcomeScreenPanel(wxWindow *parent)
     : WelcomeScreenBase(parent)
 {
     auto sizer = new wxBoxSizer(wxVERTICAL);
     auto uberSizer = new wxBoxSizer(wxHORIZONTAL);
     uberSizer->AddStretchSpacer();
-    uberSizer->Add(sizer, wxSizerFlags().Center().Border(wxALL, 50));
+    uberSizer->Add(sizer, wxSizerFlags().Center().Border(wxALL, PX(50)));
     uberSizer->AddStretchSpacer();
     SetSizer(uberSizer);
 
@@ -185,28 +206,37 @@ WelcomeScreenPanel::WelcomeScreenPanel(wxWindow *parent)
     auto header = new HeaderStaticText(this, wxID_ANY, _("Welcome to Poedit"));
     header->SetFont(m_fntHeader);
     header->SetForegroundColour(m_clrHeader);
-    sizer->Add(header, wxSizerFlags().Center().Border(wxTOP, 10));
+    sizer->Add(header, wxSizerFlags().Center().Border(wxTOP, PX(10)));
 
     auto version = new wxStaticText(this, wxID_ANY, wxString::Format(_("Version %s"), wxGetApp().GetAppVersion()));
     version->SetFont(m_fntSub);
     version->SetForegroundColour(m_clrSub);
     sizer->Add(version, wxSizerFlags().Center());
 
-    sizer->AddSpacer(20);
+    sizer->AddSpacer(PX(20));
 
     sizer->Add(new ActionButton(
                        this, wxID_OPEN,
                        _("Edit a translation"),
                        _("Open an existing PO file and edit the translation.")),
-               wxSizerFlags().Border().Expand());
+               wxSizerFlags().PXBorderAll().Expand());
 
     sizer->Add(new ActionButton(
                        this, XRCID("menu_new_from_pot"),
                        _("Create new translation"),
                        _("Take an existing PO file or POT template and create a new translation from it.")),
-               wxSizerFlags().Border().Expand());
+               wxSizerFlags().PXBorderAll().Expand());
 
-    sizer->AddSpacer(50);
+#ifdef HAVE_HTTP_CLIENT
+    sizer->Add(new ActionButton(
+                       this, XRCID("menu_open_crowdin"),
+                       _("Collaborate on a translation with Crowdin"),
+                       _("Download a file from Crowdin project, translate and sync your changes back.")),
+               wxSizerFlags().PXBorderAll().Expand());
+    sizer->Add(new LearnAboutCrowdinLink(this, _("What is Crowdin?")), wxSizerFlags().Right().Border(wxRIGHT, PX(8)));
+#endif // HAVE_HTTP_CLIENT
+
+    sizer->AddSpacer(PX(50));
 }
 
 
@@ -218,14 +248,14 @@ EmptyPOScreenPanel::EmptyPOScreenPanel(PoeditFrame *parent)
     auto sizer = new wxBoxSizer(wxVERTICAL);
     auto uberSizer = new wxBoxSizer(wxHORIZONTAL);
     uberSizer->AddStretchSpacer();
-    uberSizer->Add(sizer, wxSizerFlags().Center().Border(wxALL, 100));
+    uberSizer->Add(sizer, wxSizerFlags().Center().Border(wxALL, PX(100)));
     uberSizer->AddStretchSpacer();
     SetSizer(uberSizer);
 
     auto header = new HeaderStaticText(this, wxID_ANY, _(L"There are no translations. That’s unusual."));
     header->SetFont(m_fntHeader);
     header->SetForegroundColour(m_clrHeader);
-    sizer->Add(header, wxSizerFlags().Center().Border());
+    sizer->Add(header, wxSizerFlags().Center().PXBorderAll());
 
     auto explain = new wxStaticText(this, wxID_ANY, _("Translatable entries aren't added manually in the Gettext system, but are automatically extracted\nfrom source code. This way, they stay up to date and accurate.\nTranslators typically use PO template files (POTs) prepared for them by the developer."));
     explain->SetFont(m_fntNorm);
@@ -234,29 +264,29 @@ EmptyPOScreenPanel::EmptyPOScreenPanel(PoeditFrame *parent)
 
     auto learnMore = new wxHyperlinkCtrl(this, wxID_ANY, _("(Learn more about GNU gettext)"), "http://www.gnu.org/software/gettext/manual/");
     learnMore->SetFont(m_fntNorm);
-    sizer->Add(learnMore, wxSizerFlags().Border(wxTOP|wxBOTTOM).Align(wxALIGN_RIGHT));
+    sizer->Add(learnMore, wxSizerFlags().PXBorder(wxTOP|wxBOTTOM).Align(wxALIGN_RIGHT));
 
     auto explain2 = new wxStaticText(this, wxID_ANY, _("The simplest way to fill this catalog is to update it from a POT:"));
     explain2->SetFont(m_fntNorm);
     explain2->SetForegroundColour(m_clrNorm);
-    sizer->Add(explain2, wxSizerFlags().DoubleBorder(wxTOP));
+    sizer->Add(explain2, wxSizerFlags().PXDoubleBorder(wxTOP));
 
     sizer->Add(new ActionButton(
                        this, XRCID("menu_update_from_pot"),
                        _("Update from POT"),
                        _("Take translatable strings from an existing POT template.")),
-               wxSizerFlags().Border().Expand());
+               wxSizerFlags().PXBorderAll().Expand());
 
     auto explain3 = new wxStaticText(this, wxID_ANY, _("You can also extract translatable strings directly from the source code:"));
     explain3->SetFont(m_fntNorm);
     explain3->SetForegroundColour(m_clrNorm);
-    sizer->Add(explain3, wxSizerFlags().DoubleBorder(wxTOP));
+    sizer->Add(explain3, wxSizerFlags().PXDoubleBorder(wxTOP));
 
     auto btnSources = new ActionButton(
                        this, wxID_ANY,
                        _("Extract from sources"),
                        _("Configure source code extraction in Properties."));
-    sizer->Add(btnSources, wxSizerFlags().Border().Expand());
+    sizer->Add(btnSources, wxSizerFlags().PXBorderAll().Expand());
 
     btnSources->Bind(wxEVT_BUTTON, [=](wxCommandEvent&){
         parent->EditCatalogPropertiesAndUpdateFromSources();
